
# This file was automatically generated by Dist::Zilla::Plugin::ModuleBuild v6.008.
use strict;
use warnings;

use Module::Build 0.3601;


my %module_build_args = (
  "build_requires" => {
    "Module::Build" => "0.3601"
  },
  "configure_requires" => {
    "Module::Build" => "0.3601"
  },
  "dist_abstract" => "generate svg trees for given treebank",
  "dist_author" => [
    "Maty\x{e1}\x{161} Kopp <matyas.kopp\@gmail.com>"
  ],
  "dist_name" => "PMLTQ-Command-printtrees",
  "dist_version" => "0.0.1",
  "license" => "perl",
  "module_name" => "PMLTQ::Command::printtrees",
  "recursive_test_files" => 1,
  "requires" => {
    "File::Basename" => 0,
    "File::Path" => 0,
    "File::ShareDir" => "1.102",
    "File::Which" => "1.22",
    "Hash::Merge" => 0,
    "PMLTQ" => "v1.4.0"
  },
  "share_dir" => {
    "dist" => "share"
  },
  "test_requires" => {
    "File::Spec" => 0,
    "IO::Handle" => 0,
    "IPC::Open3" => 0,
    "Test::More" => 0
  }
);


my %fallback_build_requires = (
  "File::Spec" => 0,
  "IO::Handle" => 0,
  "IPC::Open3" => 0,
  "Module::Build" => "0.3601",
  "Test::More" => 0
);


unless ( eval { Module::Build->VERSION(0.4004) } ) {
  delete $module_build_args{test_requires};
  $module_build_args{build_requires} = \%fallback_build_requires;
}

my $build = Module::Build->new(%module_build_args);


$build->create_build_script;
